/*-
 * Copyright (c) 2010 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <codecs.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

static const char *leet[] = {
	"4",
	"8",
	"(",
	"|)",
	"3",
	"|=",
	"6",
	"|-|",
	"!",
	"_|",
	"X",
	"1",
	"/\\/\\",
	"|\\|",
	"0",
	"|*",
	"0_",
	"|2",
	"5",
	"7",
	"|_|",
	"\\/",
	"\\/\\/",
	"%",
	"j",
	"2"
};

/* convert alphabetic chars to the leet char set -- see above */
int
toleet(const char *in, const size_t insize, const char *op, void *vp, size_t outsize)
{
	const char	*cp;
	size_t		 i;
	size_t		 o;
	char		*out = (char *)vp;

	for (i = 0, o = 0 ; i < insize && o < outsize - 1 ; i++) {
		if (isalpha((uint8_t)in[i])) {
			cp = leet[tolower((uint8_t)in[i]) - 'a'];
			(void) memcpy(&out[o], cp, strlen(cp));
			o += strlen(cp);
		} else {
			out[o++] = in[i];
		}
	}
	out[o] = 0x0;
	return (int)o;
}

int
main(int argc, char **argv)
{
	codecs_t	 codecs;
	char		 buf[BUFSIZ];
	int		 cc;

	(void) memset(&codecs, 0x0, sizeof(codecs));
	codecs_add(&codecs, "leet", toleet, "500%", 1);
	for (;;) {
		(void) fprintf(stderr, "Leet> ");
		/* thanks, yes, i know, this is superfluous */
		(void) fflush(stderr);
		if (fgets(buf, sizeof(buf), stdin) == NULL) {
			break;
		}
		cc = codecs_inplace_transform(&codecs, buf, strlen(buf), "leet");
		if (cc <= 0) {
			break;
		}
		printf("%s", buf);
	}
	exit(EXIT_SUCCESS);
}
